package de.unihannover.orderingExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;

public class ExtractAllDefectData {

    public static void main(String[] args) throws Exception {
        try {
            print(AnalyzeExperimentLogs.getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    private static void print(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        sessions.removeIf(SessionData::isJeditIncomplete);
        sessions.removeIf(SessionData::isDuplicateParticipation);
        sessions.removeIf((SessionData d) -> !d.isRealEffort(2));
        sessions.removeIf((SessionData d) -> !d.isRealEffort(1));
        sessions.removeIf((SessionData d) -> !d.isRealEffort(0));

        System.out.println("Remaining sessions " + sessions.size());

        System.out.println("pId;wmspanScore;profDevExp;javaExp;programPract;reviewPract;workStartHours;fitness;jedit;prize;controlledSetting;viewHeight;reviewNumber;treatmentCombination;treatment;patch;defect;delocalized;found;timeFound");
        for (final SessionData s : sessions) {
            for (final Defect d: WU.values()) {
                printDefectData(s, d);
            }
            for (final Defect d: P1.values()) {
                printDefectData(s, d);
            }
            for (final Defect d: P2.values()) {
                printDefectData(s, d);
            }
        }
    }

    private static void printDefectData(SessionData s, Defect d) {
        System.out.print("p" + s.getId() + ";");
        System.out.print(s.getWMSpanScore() + ";");
        System.out.print(s.getVariable("profDevExp") + ";");
        System.out.print(s.getVariable("javaExp") + ";");
        System.out.print(s.getVariable("programPract") + ";");
        System.out.print(s.getVariable("reviewPract") + ";");
        System.out.print(AnalyzeExperimentLogs.toNumber(s.getVariable("workStartHours")) + ";");
        System.out.print(s.getVariable("fitness") + ";");
        System.out.print(s.getVariable("jedit", "") + ";");
        System.out.print(s.isPrize() + ";");
        System.out.print(s.isControlledSetting() + ";");
        System.out.print(s.getViewHeight() + ";");

        final int reviewNumber = s.getReviewNumberWith(d);
        System.out.print(reviewNumber + ";");
        System.out.print(s.getTreatmentCombination() + ";");
        System.out.print(s.getTreatment(reviewNumber) + ";");
        System.out.print(s.getChange(reviewNumber) + ";");
        System.out.print(d.getShortString() + ";");
        System.out.print(d.isDelocalized() + ";");
        final int found = s.foundDefect(reviewNumber, d);
        System.out.print(found + ";");
        System.out.println(found == 1 ? s.getNetTimeUntilFound(reviewNumber, d) : s.getNetTimeForReview(reviewNumber, true));
    }

}
