package de.unihannover.orderingExperimentAnalysis;

import static de.unihannover.orderingExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;
import de.unihannover.orderingExperimentAnalysis.SessionData.FP;

public enum P2 implements Defect {
    /**
     * Missing check for end < text.length in isWholeWord (hunk q).
     */
    MISSING_CHECK_END,
    /**
     * Missing check for start != 0 in isWholeWord (hunk q).
     */
    MISSING_CHECK_START,
    /**
     * Use this.noWordSep == null instead of noWordSep == null in SearchMatcher.setNoWordSep (hunk o).
     */
    SET_NO_WORD_SEP_WRONG,
    /**
     * Wrongly uses == instead of != in isEndWord (hunk r).
     */
    IS_END_WORD_WRONG_RESULT,
    /**
     * Use of wrong property name "wordBreakChars" in SearchAndReplace._replace (hunk g).
     */
    WRONG_PROPERTY,
    /**
     * Forget setting match.end in BoyerMooreSearchMatcher.nextMatch (hunk m).
     */
    MATCH_END_NOT_SET,
    /**
     * Missing continue in PatternSearchMatcher.nextMatch, leading to wrong checks (hunk j).
     */
    MISSING_CONTINUE,
    /**
     * Typo in Message "Mathc" (hunk t).
     */
    TYPO_MATCH,
    /**
     * Call to wrong constructor of PatternSearchMatcher in SearchAndReplace.getSearchMatcher() (hunk e)
     */
    WHOLEWORDMISSING,
    /**
     * Recursion in BoyerMooreSearchMatcher.nextMatch, can lead to stackoverflow (hunk m).
     */
    DANGEROUS_RECURSION;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return this == WRONG_PROPERTY || this == MATCH_END_NOT_SET || this == WHOLEWORDMISSING;
    }

    static Map<String, List<Defect>> getTrueDefects() {
        final Map<String, List<Defect>> ret = new LinkedHashMap<>();
        putDef(ret, "#aR;699;hyperSearch is not set?", FP.FP);
        putDef(ret, "#aR;700;searchSelection: where does it come from?", FP.FP);
        putDef(ret, "#aR;702;hyperSearch: where does it come from?", FP.FP);
        putDef(ret, "#aR;705;vermutlich Bedeutung vertauscht", FP.FP); //is OK
        putDef(ret, "#aR;705;too much things to review. The commit should be splitted in many. It would be easier to see each point clearly", FP.FP);
        putDef(ret, "#aR;705;I think here is a not missing? Otherwise you will always searchForward", FP.FP);
        putDef(ret, "#aR;707;geht einfacher hinzuschreiben", FP.FP); //code style
        putDef(ret, "#aR;708;How do you enable wholeWord search now?", FP.FP); //always enabled
        putDef(ret, "#aR;708;How is whole word search enabled?", FP.FP); //always enabled
        putDef(ret, "#aR;708;nothing sets whole word search to false.", FP.FP);
        putDef(ret, "#aR;708;Why?", FP.FP); //because whole word search is now always possible
        putDef(ret, "#aR;709;Why was this removed?", FP.FP); //because whole word search is now always possible
        putDef(ret, "#aL;709;Warum ist das weggefallen? Der PatternSearchMatcher unterstützt immer noch keinen wholeWord-Modus.", FP.FP);
        putDef(ret, "#aL;709;What is this change doing? To me it seems unrelated to the feature", FP.FP); //no, it is the UI side of the feature
        putDef(ret, "#aL;709;Make sure that this is true by default", FP.FP); //it is
        putDef(ret, "#aR;712;fehlt hier ggf. wholeWord-Behandlung", FP.FP);
        putDef(ret, "#bR;50;Unnecessary/redundant field? noWordSep is passed to SearchMatcher which uses.", FP.FP);
        putDef(ret, "#bR;50;could be inconsistent, why is it double stored?", FP.FP);
        putDef(ret, "#bR;50;soll final sein", FP.FP); //code style
        putDef(ret, "#bR;50;? Why specifiy here, its a property SearchMatcher.", FP.FP); //?
        putDef(ret, "#bR;50;this is not need given that search matcher has the nowordspe as well.", FP.FP); //design issue (if correct at all)
        putDef(ret, "#cR;54;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#cR;56;weil hier nicht über den Getter zugegriffen wird, hat der dort programmierte Seiteneffekt keine Auswirkung", FP.FP); //?
        putDef(ret, "#cR;57;should be this,noWordSep", FP.FP); //code style
        putDef(ret, "#cR;57;this. fehlt", FP.FP); //code style
        putDef(ret, "#cR;57;Missing \"this\"", FP.FP); //code style
        putDef(ret, "#cR;57;Might be uninitialized!", FP.FP);
        putDef(ret, "#cR;57;better use this.noWordSep", FP.FP); //code style
        putDef(ret, "#cR;57;Might be uninitialized/default value is missing!", FP.FP); //?
        putDef(ret, "#cR;57;there is no point setting this variable since a matcher is already set.", FP.FP);
        putDef(ret, "#cR;57;Klassenvariablen einheitlich mit this oder ohne this.", FP.FP); //code style
        putDef(ret, "#cR;57;why do you reference \"this.\" in the first 2 but not for noWordSep or searchMatcher?", FP.FP); //code style
        putDef(ret, "#dR;226;getter hat Seiteneffekt", FP.FP); //design issue
        putDef(ret, "#dR;227;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#dR;228;Das Feld ist doch bereits in HyperSearchOperastion Node enthalten", FP.FP);
        putDef(ret, "#dR;228;Should be \"The HyperSearchOperationNode...\"", FP.FP); //documentation
        putDef(ret, "#dR;228;That should be in the docstring. Also the has to remember should be in general revised. Let's meet later for a coffee and we restructure the documentation.", FP.FP); //documentation
        putDef(ret, "#dR;228;Is a getter method the best place to put this?", FP.FP); //design issue
        putDef(ret, "#dR;230;the nowordsep must already have been set. this is redundant in this place.", FP.FP);
        putDef(ret, "#dR;232;where does \"noWordSep\" come from?", FP.FP); //is an attribute, no problem
        putDef(ret, "#dR;232;noWordSep is not initialized, should be this.searchMatcher", FP.FP);
        putDef(ret, "#dR;232;noWordSep is not initialized", FP.FP);
        putDef(ret, "#dR;232;dirty assignment", FP.FP); //design issue
        putDef(ret, "#dR;232;set in get function", FP.FP); //design issue
        putDef(ret, "#dR;232;Where does this noWordSep come from? It doesn't seem to be accesse through `this` and it is not an argument of the function", FP.FP);
        putDef(ret, "#dR;232;state of previous returned state macther gets changed when requesting a new Search Matcher. create new search Mather with the correct property.", FP.FP); //design issue
        putDef(ret, "#dR;232;noWordSep* is not a property of the SearchMatcher..we are returning a SearchMatcher...", FP.FP); //is an attribute, no problem
        putDef(ret, "#dR;232;Where does this noWordSep come from? It doesn't seem to be accesse through `this` and it is not an argument of the function. You should probably use getNoWordSep()", FP.FP); //attribute, just used without this
        putDef(ret, "#dR;232;why not use org/experiment/editor/search/SearchMatcher.java, getNoWordSep() instead?", FP.FP); //design issue
        putDef(ret, "#dR;232;wtf?", FP.FP); //design issue
        putDef(ret, "#dR;232;setting the Separator should not be done in a this getterMethod. It should have its own method, since this can alter the behaviour of the SearchMatcher if getSearchMatcher is called againg but a new noWordSep was set", FP.FP); //design issue
        putDef(ret, "#dR;232;no logic in getters. Is it really necessary to re-use the searchMatcher instance for several buffers?", FP.FP); //design issue
        putDef(ret, "#dR;232;should be this.noWordSep, should be this.searchMatcher", FP.FP); //code style
        putDef(ret, "#dR;232;Kommentare ujnd diese Zeile löschen. Get-Methode soll normalerweise was liefern, aber nicht setzen u. s.w.", FP.FP); //design issue
        putDef(ret, "#dR;233;isn't \"this.\" needed?", FP.FP); //code style
        putDef(ret, "#eL;327;Why the missing minus?", FP.FP);
        putDef(ret, "#eR;311;wird für REs nicht geprüft", FP.FP);
        putDef(ret, "#eR;318;Should be IllegalArgumentException not just a general Exception", FP.FP);
        putDef(ret, "#eR;319;löschen", FP.FP); //code seems to be OK
        putDef(ret, "#eR;320;löschen", FP.FP); //code seems to be OK
        putDef(ret, "#eR;322;\"\".equals(string) is problematic. Should be search.isEmpty()", FP.FP);
        putDef(ret, "#eR;326;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#eR;329;noWordSep sollte gesetzt werden", P2.WHOLEWORDMISSING);
        putDef(ret, "#eR;329;RE und whole word schließen sich aus??", P2.WHOLEWORDMISSING);
        putDef(ret, "#eR;329;Hier wird der alte Konstruktor ohne wholeWord-Parameter genutzt. Der Parameter muss übergeben werden.", P2.WHOLEWORDMISSING);
        putDef(ret, "#eR;329;Hier wird der alte Konstruktor ohne wholeWord-Parameter genutzt.", P2.WHOLEWORDMISSING);
        putDef(ret, "#eR;329;wholeWord property is not set", P2.WHOLEWORDMISSING);
        putDef(ret, "#eR;329;The method getFlag is not used to retrieve the flag as before. Can be a bug depends on the Object ignoreCase", P2.WHOLEWORDMISSING); //with some goodwill
        putDef(ret, "#eR;332;You have to return a SearchMatcher.", FP.FP); //and it is returned, see no problem here
        putDef(ret, "#eR;332;Ist der BoyerMooreSearchMatcher kein gängiges Verfahren? Warum kann er auf einmal mehr als vorher?", FP.FP);
        putDef(ret, "#eR;332;Better to name the new matcher after what it really does. Not everybody is as intelligent as we are.", FP.FP);
        putDef(ret, "#eR;333;Constructor has only two parameters", FP.FP);
        putDef(ret, "#eR;333;noWordSep sollte gesetzt werden", FP.FP);
        putDef(ret, "#eR;333;The method getFlag is not used to retrieve the flag as before. Can be a bug depends on the Object ignoreCase", P2.WHOLEWORDMISSING); //with some goodwill
        putDef(ret, "#eR;335;Also you could just have returned the object. No variable needed.", FP.FP); //code style
        putDef(ret, "#fR;654;Das property ist käse, am besten in eigener Methode verpacken.", FP.FP); //design issue
        putDef(ret, "#fR;654;I guess, we already set that in `getSearchMatcher` unless I misunderstood something", FP.FP);
        putDef(ret, "#fR;655;andere Property als oben", P2.WRONG_PROPERTY);
        putDef(ret, "#fR;655;Different literal used to retrieve the same property then above. I think this literal is correct", P2.WRONG_PROPERTY);
        putDef(ret, "#fR;655; use String methods indeed, to maintain the readability of the code", P2.WRONG_PROPERTY); //interpreting string methods as string constants and applying some goodwill
        putDef(ret, "#fR;655;noWordSep soll static sein", FP.FP); //code style
        putDef(ret, "#fR;655;Is the name of the property \"wordBreakChars\" or \"noWordSep\" (see replace() line 1195)", P2.WRONG_PROPERTY);
        putDef(ret, "#fR;655;are there 2 properties for that?", P2.WRONG_PROPERTY);
        putDef(ret, "#fR;655;since string \"noWordSep\" is used many times, isn't it better to introduce a constant?", FP.FP);
        putDef(ret, "#fR;660;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#gR;1187;missing throws in javadoc", FP.FP); //code style
        putDef(ret, "#gR;1192;", FP.FP);
        putDef(ret, "#gR;1193;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#gR;1194;You cast here. The library is not good to allow this. Be careful and catch the maybe occuring error.", FP.FP); //design issue
        putDef(ret, "#gR;1194;Warum werden nicht die Chars aus SearchMatcher verwendet?", FP.FP);
        putDef(ret, "#gR;1194;muss es nicht noWordSep sein?", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1194;Warum werden nicht die Chars aus SearchMatcher verwendet? Das property ist käse, am besten in eigener Methode verpacken.", P2.WRONG_PROPERTY); //well, not really
        putDef(ret, "#gR;1194;wrong property or wrong setter in the following expression", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;Property is named noWordSep!", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;soll static string sein", P2.WRONG_PROPERTY); //line is correct but comment is code style, we count it anyway
        putDef(ret, "#gR;1195;should be noWordSep", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;Literals should have a constant class or be constants in this class. Since this literal is different to the literal below to retrieve the nowordsep. I think the one below is correct.", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;property should be \"noWordSep\"", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195; use String methods indeed", FP.FP);
        putDef(ret, "#gR;1195;Is the name of the property \"wordBreakChars\" or \"noWordSep\"", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;Is the name of the property \"wordBreakChars\" or", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;The property is called `noWordSep` elsewhere", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;should this not be property noWordSep instead of wordBreakChars?", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;Is the name of the property \"wordBreakChars\" or \"noWordSep\" (see find() line 655)", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;wordBreakChars used for noWordSep", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195;Literals should have a constant class or be constants in this class.", P2.WRONG_PROPERTY); //with goodwill
        putDef(ret, "#gR;1195;Die Property heißt noWordSep, nicht wordBreakChars", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1195; use String methods indeed, to maintain the readability of the code", P2.WRONG_PROPERTY); //interpreting string methods as string constants and applying some goodwill
        putDef(ret, "#gR;1196;setNoWordSep() expectes word characters that are not separators, while \"wordBreakChars\" seems to be the opposite (i.e. a set of separators)", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1196;setNoWordSep wird hier nie wieder zurückgesetzt. Anstatt mit einem Setter (mutable state) zu arbeiten, sollte noWordSep lieber konsequent als Parameter übergeben werden.", FP.FP); //design issue
        putDef(ret, "#gR;1196;not 100% sure about this but aren't the \"noWordSep\" the opposite of \"workBreakChars\"?", P2.WRONG_PROPERTY);
        putDef(ret, "#gR;1196;Does this work? Shouldn't you set the number of chars and not the chars themselves?", FP.FP);
        putDef(ret, "#gR;1196;Vielleicht kein Fehler, ab setNoWordSep wird hier nicht wieder zurückgesetzt. Anstatt mit einem Setter (mutable state) zu arbeiten, sollte noWordSep lieber konsequent als Parameter übergeben werden.", FP.FP);
        putDef(ret, "#gR;1203;avoid labels. condition is also missing in this for loop", FP.FP); //code style
        putDef(ret, "#gR;1204;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#gR;1222;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#hR;55;Überflüssiger Hinweis", FP.FP); //documentation
        putDef(ret, "#hR;62;re ist nicht sprechend, wholeWord sollte einen Schalter beschreiben: useWholWord o.Ä.", FP.FP); //documentation
        putDef(ret, "#hR;63;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#hR;64;check init of re", FP.FP); //is OK
        putDef(ret, "#hR;64;Pattern.pattern() kommt mir gerade komisch vor. Wir haben schonnnn das Pattern und merken uns die Zeichen davon nochmal extra?", FP.FP); //is OK
        putDef(ret, "#hR;64;Is this constructor invocation really needed?", FP.FP); //yes, it is (at least I see no reason why it shouldnt)
        putDef(ret, "#hR;64;This causes an infinite loop (actually finite ending with a stackoverflow", FP.FP); //no, is OK
        putDef(ret, "#hR;64;Pattern.pattern() kommt mir gerade komisch vor", FP.FP);
        putDef(ret, "#hR;65;re und re.pattern werden beide hier genutzt?", FP.FP);
        putDef(ret, "#hR;65;the previous constructor call is already setting the pattern, this is overriding that setting.", FP.FP); //?
        putDef(ret, "#iR;76;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#iR;77;Standardwert false dokumentieren.", FP.FP); //documentation
        putDef(ret, "#iR;77;whole word search is always disabled", FP.FP);
        putDef(ret, "#iR;77;You should add the information that this constructor does not support the whole word search", FP.FP); //documentation
        putDef(ret, "#iR;77;false löschen", FP.FP); //wrong
        putDef(ret, "#iR;77;cyclic dependency with other constructor", FP.FP); //no, it isn't, a third (unchanged) constructor is called
        putDef(ret, "#iR;77;missing line: this.re = re;", FP.FP); //is set in other constructor
        putDef(ret, "#jR;85;Dokumentation is lacking alot", FP.FP); //documentation
        putDef(ret, "#jR;87;Diese Methode ist viel zu groß, refaktorisieren.", FP.FP); //design issue
        putDef(ret, "#jR;87;Die Methode ist völlig unübersichtlich", FP.FP); //design issue
        putDef(ret, "#jR;87;This whole methos is way to long. It is unclear what it is doing, Should be refactored into smaller methods.", FP.FP);
        putDef(ret, "#jR;87;dot", FP.FP);
        putDef(ret, "#jR;100;Was tut das hier?", FP.FP);
        putDef(ret, "#jR;101;input parameters should not be overriden. A new instance of CharSequence should be created", FP.FP); //design issue
        putDef(ret, "#jR;104;Hierfür brauchten wir das Extra-Pattern aus dem Konstruktor?", FP.FP);
        putDef(ret, "#jR;112;Warum ist der erste Fund nicht korrekt? Soll \"^\" nur ab der zweiten Zeile matchen?", FP.FP); //will be probably be as intended, when there is so much description about it
        putDef(ret, "#jR;155;skips \"whole word\" checking below", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;159;beim ersten Durchlauf kommt ggf. immer null? Weril pevious mit null initialisiert ist", FP.FP);
        putDef(ret, "#jR;159;beim ersten Durchlauf kommt ggf. immer null?", FP.FP);
        putDef(ret, "#jR;163;hier wird der soeben in der Schleife gesetzte Wert überschrieben", FP.FP);
        putDef(ret, "#jR;175;wholeWord not initialized", FP.FP); //wholeWord is an attribute and initialized somewhere else
        putDef(ret, "#jR;175;Ich meine Pattern-Matching und Ganzwortsuche schlossen sich vorn aus", FP.FP); //that was before the change
        putDef(ret, "#jR;175;isWholeWord has to be called with consecutive chars at beginning and end of word", FP.FP); //wrong
        putDef(ret, "#jR;175;The variable \"wholeWord\" is a bit unhappy named. I think it would be better to be something like \"wholeWordActive\".", FP.FP); //naming
        putDef(ret, "#jR;175;This logic does not properly update returnValue.start and returnValue.end.", P2.MISSING_CONTINUE); //perhaps?
        putDef(ret, "#jR;175;this. fehlt", FP.FP); //code style
        putDef(ret, "#jR;175;I’m not really sure what’s going here, but what if the next match is also not a whole word? Looks like this needs to be something like `while`...", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;175;I am confused why are we asking two times the same question? Also one if is enough.", FP.FP); //I don't see a repeated question there
        putDef(ret, "#jR;175;I am confused why are we asking two times the same question?", FP.FP);
        putDef(ret, "#jR;175;the two conditions appear to cancel each other out.", FP.FP); //no, one activates whole word search, the other checks the condition
        putDef(ret, "#jR;175;Where comes \"wholeWord\" from, is there a variable missing?", FP.FP); //code style
        putDef(ret, "#jR;175;wholeWord existiert nicht?", FP.FP);
        putDef(ret, "#jR;176;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#jR;177;if Match.find() is successful, its result is not checked again for the \"whole word\" property", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;not clear here. should we not be calling nextMatch() recursively here?", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;shouldn't be exiting loop here", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;position of check questionable as the checks and substitution group assignments above are not redone after recusive call to Match.find()", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;position of check questionable as the checks above are not redone after recusive call to Match.find()", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;here recursion or loop is necessary to fulfill the requirements. Searches first candidate, then second, but then breaks as \"not found\", which isn't correct.", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;177;Wenn !isWholeWird und es das letzte Vorkommen des Wortes ist, wird korrekt null zurückgegeben, aber returnValue wird nicht wieder zurückgesetzt (keine Ahnung, wer den alles ausliest).", FP.FP);
        putDef(ret, "#jR;177;Curly braces", FP.FP); //Code style
        putDef(ret, "#jR;177;Wenn !isWholeWird und es das letzte Vorkommen des Wortes ist, wird korrekt null zurückgegeben, aber returnValue wird nicht wieder zurückgesetzt.", FP.FP);
        putDef(ret, "#jR;178;loop must be repeated to find the new match, _start and_end are not updated.", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;178;Bug candidate. If two candidates in the row are not whole words, this returns null. These are the literal requirements, but this really sounds like a mistake and one would assume it should loop until a satisfying candidate is found or all are exhausted.", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;179;Was, wenn match.find() doch etwas zurückgibt? Jetzt zeigt returnValue auf das alte nicht-wholeWord-kompatible Vorkommen", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;184;In case of reverse we keep doing `find` in this loop until we find the last match, but we no longer check that it is a whole word...", P2.MISSING_CONTINUE);
        putDef(ret, "#jR;226;Aufwand für Review wird zu groß. Di Klasse muss refaktorisiert werde. Tut mit ziemlicher Sicherheit nicht das, was sie soll.", FP.FP); //design issue
        putDef(ret, "#kR;36;Was zeichnet den BoyerMooreSerahcMatcher aus? Das ist doch sicherlich ein Standard-Algorithmus, näher beschreiben.", FP.FP); //documentation
        putDef(ret, "#kR;41;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#kR;42;Standard \"false\" dokumnetieren.", FP.FP); //documentation
        putDef(ret, "#kR;42;this yields an infinite loop!", FP.FP);
        putDef(ret, "#kR;42;whole word search is always disabled.", FP.FP);
        putDef(ret, "#kR;42;false ist nicht verständlich", FP.FP); //documentation
        putDef(ret, "#lR;54;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#lR;56;There is a string method toUppercase if i recall correctly. Use this if you really want to make everything uppercase. After that invoke toCharArray", FP.FP); //design issue
        putDef(ret, "#lR;56;There is a string method toUppercase if i recall correctly. Use this if you really want to make everything uppercase", FP.FP);
        putDef(ret, "#lR;57;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#lR;59;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#lR;60;Das sieht erstmal nach keiner guten Idee aus (UTF-16...), kann ich aber ohne den Algorithmus dazu nicht sicher sagen", FP.FP);
        putDef(ret, "#lR;60;Das ist aus vielen Gründen (ß, UTF-16...) keine gute Idee, aber nicht Teil dieses Changesets :)", FP.FP);
        putDef(ret, "#lR;60;Das sieht erstmal nach keiner guten Idee aus (ß, UTF-16...), kann ich aber ohne den Algorithmus dazu nicht sicher sagen", FP.FP);
        putDef(ret, "#lR;60;Das sieht nach keiner guten Idee aus (ß, UTF-16...), kann man aber ohne den Algorithmus dazu nicht sicher sagen", FP.FP);
        putDef(ret, "#lR;67;patternEnd? where did this come from, why is it needed?", FP.FP); //is attribute, code style
        putDef(ret, "#lR;67;-1 löschen", FP.FP);
        putDef(ret, "#lR;67;Und bei einem leeren Pattern-String?", FP.FP);
        putDef(ret, "#lR;53;Schalter artig benennen: matchWholeWord o.Ä.", FP.FP); //code style
        putDef(ret, "#lR;55;no null checks on pattern", FP.FP); //code style
        putDef(ret, "#mR;78;besser pos < 0", FP.FP); //code style
        putDef(ret, "#mR;83;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#mR;84;returnValue not initialized", FP.FP); //returnValue is an attribute and initialized somewhere else
        putDef(ret, "#mR;84;Moment, woher kommt die Variable \"returnValue\"? Wenn das ein Feld der Klasse BoyerMooreSearchMatcher ist, muss es wahrscheinlich auch im folgenden if-Block gesetzt werden.", FP.FP);
        putDef(ret, "#mR;84;this.returnValue", FP.FP); //code style
        putDef(ret, "#mR;84;Where is returnValue created? Outside of this method? That is very \"unusual\"", FP.FP); //design issue
        putDef(ret, "#mR;84;int returnValue = ...", FP.FP);
        putDef(ret, "#mR;84;\"ReturnValue\" not local prone to errors, the recursive call of \"nextMatch\" can change it", FP.FP); //design issue
        putDef(ret, "#mR;84;Moment, woher kommt die Variable \"returnValue\"? Wenn das ein Feld der Klasse BoyerMooreSearchMatcher ist und später nochmal verwendet wird, muss es wahrscheinlich auch im folgenden if-Block angepasst werden.", FP.FP); //design issue
        putDef(ret, "#mR;84;Moment, woher kommt die Variable \"returnValue\"? Wenn das ein Feld der Klasse BoyerMooreSearchMatcher ist, muss es wahrscheinlich auch im folgenden if-Block angepasst werden.", FP.FP);
        putDef(ret, "#mR;85;this is incorrect if \"pattern\" can consist of wildcards or similar, as then the length of the pattern does not correspond to the length of the character sequence found", FP.FP);
        putDef(ret, "#mR;85;vermutlich einen zu weit gezählt", FP.FP);
        putDef(ret, "#mR;86;Warum eine Extra-Variable _end? Das steht auch in returnValue", FP.FP); //design issue
        putDef(ret, "#mR;86;Don't name with an underscore. Just substitute in the expression below.", FP.FP); //design issue
        putDef(ret, "#mR;86;", FP.FP);
        putDef(ret, "#mR;87;wholeWord not initialized", FP.FP); //wholeWord is an attribute and initialized somewhere else
        putDef(ret, "#mR;87;isWholeWord should be compared for true not false", FP.FP); //wrong
        putDef(ret, "#mR;87;wholeWord is set to false in the constructor", FP.FP);
        putDef(ret, "#mR;87;this.wholeWord", FP.FP); //code style
        putDef(ret, "#mR;87;in order for isWholeWord to have a chance of working properly one would have to pass _end+1", FP.FP);
        putDef(ret, "#mR;87;Die Rekursion hier kann leicht zu einem StackOverflow führen. Lieber iterativ lösen und die Verschachtelung an dieser Stelle auflösen.", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;87;wtf is _end?", FP.FP);
        putDef(ret, "#mR;88;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#mR;89;es wird nicht garantiert, dass der neue match ein ganzes word ist", FP.FP); //is guaranteed through recursive call
        putDef(ret, "#mR;89;not sure about subSequence() api here: bug if second param must be within bounds of subject", FP.FP); //is correct, end index is exclusive
        putDef(ret, "#mR;89;Double check: you could be missing a -1 after test.length() or a +1 after _end", FP.FP); //is correct, end index is exclusive
        putDef(ret, "#mR;89;IOOBE?", FP.FP);
        putDef(ret, "#mR;89;Hier wird `reverse` nicht beachtet, sondern immer *nach* dem gefundenen Text weitergesucht.", FP.FP);
        putDef(ret, "#mR;89;You may want to make sure subText is shorter than text, otherwise there is a chance of SO", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;89;out of bounds, should be text.length() - 1", FP.FP);
        putDef(ret, "#mR;89;", FP.FP);
        putDef(ret, "#mR;89;Es wäre sicherer, hier bei returnValue.start+1 statt bei returnValue.end weiterzusuchen, falls das letzte (nicht wholeWords-kompatible) und nächste wholeWords-kompatible Match sich überlappen. Falls das nicht passieren kann, bitte so begründen und als Kommentar hier einfügen.", FP.FP);
        putDef(ret, "#mR;89;Es wäre sicherer, hier bei returnValue.start+1 statt bei returnValue.end weiterzusuchen, falls das letzte (nicht wholeWords-kompatible) und nächste wholeWords-kompatible Match sich überlappen. Wenn das nicht passieren kann (kommt auf die finale Logik in isEndOfWord(...) an), dann bitte in einem Kommentar so dokumentieren.", FP.FP);
        putDef(ret, "#mR;89;Hier wird `reverse` nicht beachtet, sondern immer *nach* dem gefundenen Text weitergesucht", FP.FP);
        putDef(ret, "#mR;90;SearchMatcher.Match", FP.FP);  //would be compile error
        putDef(ret, "#mR;90;Das ist ein rekursiver Aufruf, gefährlich, geht das nicht auch irgendwie iterativ?", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;90;stack overflow error by using recursion, also requires every subsequence of text to be kept live in memory at the same time", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;90;Die Rekursion hier kann leicht zu einem StackOverflow führen. Lieber iterativ lösen und die Verschachtelung an dieser Stelle auflösen.", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;90;Will there be a stack overflow if there are many non-whole-word matches?", P2.DANGEROUS_RECURSION);
        putDef(ret, "#mR;90;we can return here itself. else set match.end  before line 95", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;90;Since returnValue is a class field, mach and returnValue are the same instance. That is very confusing and a source of errors. That should be refactored.", FP.FP);
        putDef(ret, "#mR;90;Recursive call changes the same instance variable \"returnValue\", you have to use a copy here aswell", FP.FP);
        putDef(ret, "#mR;91;unclear role of \"start\" and \"end\" parameters because they are not used anywhere", FP.FP); //could be a design issue
        putDef(ret, "#mR;91;das dritte \"end\" in verschiedenen Variationen hier - SchrottCode Grrr.", FP.FP); //design issue
        putDef(ret, "#mR;91;start should be 0 and end should be adjusted too", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;91;start, end should be updated with _end offset", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;91;start and end index are not updated", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;91;Der boolean start, der den Anfang einer Zeile markiert (wofür?) muss hier neu evaluiert werden, weil der Ausschnitt des Textes sich geändert hat", FP.FP);
        putDef(ret, "#mR;91;Der boolean start, der den Anfang einer Zeile markiert (wofür?) muss hier neu evaluiert werden", FP.FP);
        putDef(ret, "#mR;91;Die Indizes start/end stimmen nicht mehr, wenn text eine subSequence am Ende des alten Texts ist", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;91;start/end seem to be unused. Is this correct? If not, should they be changed when we're searching in subSequence?", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;91;Je nachdem was die booleans \"start\" aussagen, müssen sie hier vielleicht auf andere Werte gesetzt werden (Vorwärtssuche: start=false, Rückwärtssuche: end=false?)", FP.FP);
        putDef(ret, "#mR;92;Should be the other way round. match != null -> match.start = match.start + _end. Return match at the end of method", FP.FP); //design issue (albeit triggered by defect closeby)
        putDef(ret, "#mR;92;Curly braces", FP.FP); //code style
        putDef(ret, "#mR;94;Use +=", FP.FP); //code style
        putDef(ret, "#mR;94;+= ?", FP.FP);
        putDef(ret, "#mR;94;match.end muss ebenfalls angepasst werden", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;the index is off by 1.", FP.FP);
        putDef(ret, "#mR;94;\"match.end\" is not transformed (\"+ _end\" is needed here, too)", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;_end ist nicht relativ zu start sondern absolut, oder?", P2.MATCH_END_NOT_SET); //did the reviewer really mean this?
        putDef(ret, "#mR;94;Did you forgot to shift the match end?", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;wtf?", P2.MATCH_END_NOT_SET); // with a lot of goodwill
        putDef(ret, "#mR;94;This seems to be strange", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;You also need to adjust `match.end` the same way", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;missing match.end = ...", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;match.end needs also to be set", P2.MATCH_END_NOT_SET);
        putDef(ret, "#mR;94;I don't understand what this is supposed to do. So i cannot say if it is correct. Splitting this method up and give each part proper names might increase understandability.", P2.MATCH_END_NOT_SET); //with goodwill
        putDef(ret, "#mR;95;Leave the returnValue. Don't make new returns.", FP.FP); //code style
        putDef(ret, "#mR;95;the new result should be checked if it is a whole word before returned (if wholeWord is true)", FP.FP);
        putDef(ret, "#nR;242;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#nR;243;Das property ist käse, am besten in eigener Methode verpacken.", FP.FP); //design issue
        putDef(ret, "#nR;243;den Code sehe ich jetzt zum dritten mal", FP.FP); //design issue
        putDef(ret, "#nR;244;Is the name of the property \"wordBreakChars\" or \"noWordSep\" (see SearchAndReplace.replace() line 1195)", P2.WRONG_PROPERTY);
        putDef(ret, "#nR;244;This literal seems to be the correct one since it was used twice. Should be a constant somewhere. All this casting of variables is suspicious and might raise an Exception if the variable is noll or not a string", P2.WRONG_PROPERTY);
        putDef(ret, "#nR;244;Is the name of the property \"wordBreakChars\" or \"noWordSep\" (see replace() line 1195)", P2.WRONG_PROPERTY);
        putDef(ret, "#nR;244;This literal seems to be the correct one since it was used twice. Should be a constant somewhere", P2.WRONG_PROPERTY);
        putDef(ret, "#nR;244;noWordSep soll als static String sein", FP.FP); //code style
        putDef(ret, "#nR;245;Siehe Kommentar weiter oben: Bitte vermeiden, dass sich zwei Klassen über Setter um den SearchMatcher#noWordsSep \"streiten\".", FP.FP); //design issue
        putDef(ret, "#nR;291;Where is the catch-block to do exception handling?", FP.FP); //why should there be one?
        putDef(ret, "#oR;56;parameter name does not match description", FP.FP); //documentation
        putDef(ret, "#oR;56;*are", FP.FP); //documentation
        putDef(ret, "#oR;56;Variablenname passt nicht zur Beschreibung. Hinweis auf Fallback _ fehlt", FP.FP); //documentation
        putDef(ret, "#oR;56;Variablenname passt nicht zur Beschreibung", FP.FP);
        putDef(ret, "#oR;56;\"as no word chars\"", FP.FP); //documentation
        putDef(ret, "#oR;60;are noWordSep one or more characters? Because noWordSep is not a collection", FP.FP); //documentation
        putDef(ret, "#oR;60;parameter has the same name as a field variable", FP.FP); //code style
        putDef(ret, "#oR;61;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#oR;62;remove this", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;this should check the parameter not the field", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;the argument should be checked for null and not the field", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;What about the empty String ?", FP.FP); //empty set of word break chars seems to be OK
        putDef(ret, "#oR;62;return this.noWordSep == null? \"_\" : noWordSep", FP.FP); //code style
        putDef(ret, "#oR;62;es muss der aktualparameter überprüft werden", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;does not makes sense, should be without this.", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;Does not seem to make sense. Probably `this.` should not be there.", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;Remove this. as you should null check the incoming argument", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;shoudl comapre noWordSep not this.noWordSep", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;\"this.\" is wrong", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;don't test for instance variable, but for parameter", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;the if should check the parameter not the field", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;if(noWordSep == null)", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;this löschen", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;check should be if (noWordSep... no this.", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;Check should be on parameter (remove this.)", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;62;check should be if(noWordSep == null)", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;why is the paramter ignored in this case?", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;given parameter ignored", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;given parameter ignored, wrong assignment", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;So the argument doesn't matter? Probably extra \"this.\" should be removed in the previous line?", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;Warum ist es wichtig, dass der Setter einen Standardwert setzt (Kommentar)? Heißt das, dass das Objekt in einem ungültigen Zustand ist, bevor man zum ersten Mal setNoWordSep aufruft (weil noWordSep dann null ist, was es anscheinend nie sein soll)?", FP.FP); //design issue
        putDef(ret, "#oR;63;Warum ist es wichtig, dass der Setter einen Standardwert setzt (Kommentar)? Heißt das, dass das Objekt in einem ungültigen Zustand ist, bevor man zum ersten Mal setNoWordSep aufruft (weil noWordSep dann null ist, was es anscheinend nie sein soll)? -> Den Fallback lieber in den Getter oder in die aufrufende Methode verschieben?", FP.FP);
        putDef(ret, "#oR;63;strange choice for a default", FP.FP);
        putDef(ret, "#oR;63;how_about_this_case_?", FP.FP);
        putDef(ret, "#oR;63;the separator for the first time is set to the wrong char, this will cause the search to fail.", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;is it right that when a arg is defined but this.noWordSep is null that the value is set to \"_\"?", P2.SET_NO_WORD_SEP_WRONG);
        putDef(ret, "#oR;63;unclear; why should one ever wants to only have the underscore as non-separating character?", FP.FP); //could be an analysis error
        putDef(ret, "#oR;63;Warum ist es wichtig, dass der Setter einen Standardwert setzt? Heißt das, dass das Objekt in einem ungültigen Zustand ist, bevor man zum ersten Mal setNoWordSep aufruft (weil noWordSep dann null ist, was es anscheinend nie sein soll)? -> Den Fallback lieber in den Getter oder in die aufrufende Methode verschieben", FP.FP);
        putDef(ret, "#oR;63;Why only \"_\" and not also \"-\"?", FP.FP); //could be an analysis error
        putDef(ret, "#oR;63;Hätte ein Leerzeichen erwartet", FP.FP); //that's not the problem
        putDef(ret, "#oR;63;What about space and line end? I think they should be included in this \"list\"", FP.FP);
        putDef(ret, "#oR;65;Why don't set this.noWordSep without any check?", P2.SET_NO_WORD_SEP_WRONG); //with some goodwill
        putDef(ret, "#pR;67;Nur die erste Zeile vom Kommentare soll bleiben, der Rest löschen", FP.FP); //documentation
        putDef(ret, "#pR;68;Die Beschreibung beschreibt nicht, was der Inhalt der Zeichenkette ist. @return ist redundant.", FP.FP); //documentation
        putDef(ret, "#pR;68;No need for a method.", FP.FP); //code style
        putDef(ret, "#pR;74;muss ggf. public sein?", FP.FP);
        putDef(ret, "#pR;74;", FP.FP);
        putDef(ret, "#pR;74;package private on purpose?", FP.FP);
        putDef(ret, "#pR;74;private", FP.FP);
        putDef(ret, "#pR;75;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#pR;76;shouold be this.noWordSep", FP.FP); //code style
        putDef(ret, "#pR;76;this. fehlt", FP.FP); //code style
        putDef(ret, "#pR;76;use this.noWordSep", FP.FP); //code style
        putDef(ret, "#pR;76;\"this.\" ?", FP.FP); //code style
        putDef(ret, "#pR;76;return this.nowordsep", FP.FP); //code style
        putDef(ret, "#qR;80;Sorry, but this JavaDoc is not helpful at all. \"Check if the result is a whole word\" The result of what? What is text? The found word? The whole search buffer? Is start and end marking the start and end of the found word in the text buffer or the start and end of what?", FP.FP); //documentation
        putDef(ret, "#qR;83;Muss dokumentiert werden: Ist end inklusive oder exklusive? Der Code nimmt exklusive an (text.charAt(end) ist das erste Zeichen nach dem result)", FP.FP); //design issue
        putDef(ret, "#qR;83;Muss dokumentiert werden: Ist end inklusive oder exklusive?", FP.FP);
        putDef(ret, "#qR;83;Muss dokumentiert werden: Ist end inklusive oder exklusive? Der Code nimmt inklusive an", FP.FP);
        putDef(ret, "#qR;86;start > end, start>=0, end < text.length should be validated", FP.FP); //would be saver, but not needed obligatorily
        putDef(ret, "#qR;86;wont work for empty string", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;87;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#qR;88;return isEndWord(text.charAt(start), text.charAt(start - 1)) || isEndWord(text.charAt(end - 1), text.charAt(end)", FP.FP); //code style
        putDef(ret, "#qR;88;index may be out of bounds", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;text prüfen, ob leer ist", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;Hier sollten auch die Sonderfälle behandelt werden, dass start==0 oder end==text.length()-1 ist; dann ist es per Definition immer eine Wortgrenze. Gerade führen diese Fälle zu einer IndexOutOfBoundsException.", P2.MISSING_CHECK_START, P2.MISSING_CHECK_END);
        putDef(ret, "#qR;88;IOOBE here if start >= text.length() (may be safe to assume in context though)", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;Bei einem leeren Text können hier einige Probleme auftreten.", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;Sanitary checks are required to avoid out of bounds exceptions", P2.MISSING_CHECK_START, P2.MISSING_CHECK_END); //counting as both with some goodwill
        putDef(ret, "#qR;88;This checks for word end but should check for word beginning", P2.MISSING_CHECK_START); //perhaps?
        putDef(ret, "#qR;88;no check/assert if start could be negative", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;Hier sollten auch die Sonderfälle behandelt werden, dass start==0 oder end==text.length() ist; dann ist es per Definition immer eine Wortgrenze", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;start index should be checked for >= 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;index may be out of bounds", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;check for 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;88;missimg check if start is == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;prevChar has to be bevore start. But if start is the first char previous will be null", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;missing check if start == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;Wait what. Is this negative if `start = 0`?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;if start is <= 0, we are fetching charAt at negative index..this case should be handled differently", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;what if start is 0?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;IOOBE if start == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;if start is <= 0, we are fetching charAt at negative index..", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;star t -1 is not checked to be less than zero", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;should be end - 1", FP.FP);
        putDef(ret, "#qR;89;star t -1", FP.FP);
        putDef(ret, "#qR;89;no check if start > 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;possible \"-1\" offset at the start of file", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;what if start == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;if start is 0 then this will fail", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;start kann 0 sein, also kann hier eine IndexOutOfBoundException fliegen", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;What if start == 0?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;Can be out of bounds if start is zero, should be checked.", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;Causes exception when start == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;what if (start == 0)?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;possible IndexOutOfBoundsException", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;crashes on start = 0 (which is a valid position for a word candidate", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;What about start == 0 ?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;should check if start == 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;Could be an error, because there is no check that start-1 is a valid index", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;start == 0?", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;previous character my not exist if this is the beginning of the line", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;missing check for start <= 0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;Might throw an exception for first char in CharSequence", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;index is out-of-bounds if start==0", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;89;IndexOutOfBounds wird nicht beachtet", P2.MISSING_CHECK_START);
        putDef(ret, "#qR;90;wrong parameter order", FP.FP); //they are indeed swapped, but (the correct version of) isEndWord is symmetric
        putDef(ret, "#qR;90;wrong order of the args here", FP.FP);
        putDef(ret, "#qR;90;Here the condition for the start Word should be checked not the end word", FP.FP);
        putDef(ret, "#qR;90;Here the condition for the start Word should be checked not the end word.", FP.FP);
        putDef(ret, "#qR;90;Is the order of the parameter correct? I think they must be changed.", FP.FP);
        putDef(ret, "#qR;90;swap parameters: (prev, first)", FP.FP);
        putDef(ret, "#qR;90;inverted arguments", FP.FP);
        putDef(ret, "#qR;90;open braces might be come at end of previous line", FP.FP);
        putDef(ret, "#qR;90;Müssten hier nicht firstChar und prevChar getauscht sein?", FP.FP);
        putDef(ret, "#qR;90;Need to be symmetric \"isStartWord\"", FP.FP);
        putDef(ret, "#qR;90;parameter in wrong order", FP.FP);
        putDef(ret, "#qR;90;The parameters should be the other way around, calling isEndWord(prevChar, firstChar)", FP.FP);
        putDef(ret, "#qR;90;This works, but you should probably wrap this usage inside a method called isStartWord or similar, jsut for clarity", FP.FP); //design issue
        putDef(ret, "#qR;90;Parameter vertauscht, hebt sich ggf. bei Tests mit dem Fehler oben auf", FP.FP); //see above
        putDef(ret, "#qR;90;argument order is wrong. isEndWord(prevChar, firstChar)", FP.FP); //see above
        putDef(ret, "#qR;90;to use isEndWord, parameters have to be switched", FP.FP);
        putDef(ret, "#qR;90;for start checking better create own methode isBeginOfWord", FP.FP); //design issue
        putDef(ret, "#qR;90;Even if this does work, the naming is confusing, as we're looking for the start of a word.", FP.FP); //naming
        putDef(ret, "#qR;90;es fehlt eine isStartWord", FP.FP); //OK because isEndWord is symmetric
        putDef(ret, "#qR;90;shouldn't the check be performed on the last characters?", FP.FP); //OK because isEndWord is symmetric
        putDef(ret, "#qR;91;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#qR;95;index may be out of bounds", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;end-1 index should be checked for >= 0", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;end -1 is not checked to be less than zero", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;what if (end == 0)?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;off-by-one, end is position of last char of word, not a length", P2.MISSING_CHECK_END); //well, not really
        putDef(ret, "#qR;95;End ist das Ende des Matches,also wäre End - 1 der vorletzte Buchstabe und nicht der letzte char", FP.FP);
        putDef(ret, "#qR;95;index is out-of-bounds if end==0", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;Same for \"end\"", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;see above", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;From the usage I saw, these have to be `end - 2` and `end - 1`", FP.FP);
        putDef(ret, "#qR;95;can end be 0?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;Arraygrenzen?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;no check if end is in the allowed range", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;95;Hier müsste end stehen?", P2.MISSING_CHECK_END); //well, not really
        putDef(ret, "#qR;96;Hier müsste end + 1 stehen?", P2.MISSING_CHECK_END); //well, not really
        putDef(ret, "#qR;96;index may be out of bounds", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;end index should be checked for >= 0", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;What if end == text.length?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;should check if end >= text.length()", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;this is going to be out of bounds too", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;And what if `text` is too short or even empty?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;off-by-one, end is position of last char of word, not a length", P2.MISSING_CHECK_END); //well, not really
        putDef(ret, "#qR;96;is the end index included in the match or is it excluded?", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;96;no chek if end is in allowed range", P2.MISSING_CHECK_END);
        putDef(ret, "#qR;97;Could be simpler: return isEndWord(lastchar,nextchar)", FP.FP); //code style
        putDef(ret, "#qR;98;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#qR;102;pls refactor so the whole method reads: return isWordEnd(start, text) && isWordEnd(end, text)", FP.FP); //design issue
        putDef(ret, "#qR;102;geht davon aus, dass innerhalb von start..end nur Wort-Zeichen stehen. Das muss aber nicht so sein.", FP.FP); //works as designed
        putDef(ret, "#qR;102;start > end should also be checked..", FP.FP);
        putDef(ret, "#rR;109;was ist, wenn current am Ende des buffers ist? Wert von next == null?", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;109;can you example how this works? I don't quite understand what you are doing here", P2.IS_END_WORD_WRONG_RESULT); //with some goodwill
        putDef(ret, "#rR;109;better name rename method to isEndOfWord", FP.FP); //naming
        putDef(ret, "#rR;109;how do we account for start and end of line?", FP.FP);
        putDef(ret, "#rR;109;soll vereinfacht werden", P2.IS_END_WORD_WRONG_RESULT); //with some goodwill
        putDef(ret, "#rR;109;how do we account for start and end of line?; a better name would be isWordEnd()", FP.FP); //with some goodwill
        putDef(ret, "#rR;110;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#rR;111;return TextUtilities.getCharType(current, noWordSep) != TextUtilities.WORD_CHAR", FP.FP); //would be wrong
        putDef(ret, "#rR;112;shouldn't this be ==? If currentChar is char which donates end of end... then true?", P2.IS_END_WORD_WRONG_RESULT); //with some goodwill
        putDef(ret, "#rR;111;Shouldn't the logic be return current == WORD_CHAR && next != WORD_CHAR?", P2.IS_END_WORD_WRONG_RESULT); //with some goodwill
        putDef(ret, "#rR;112;Was genau ist ein WORD_CHAR: Ein Wort-Trennzeichen, oder Zeichen innerhalb eines Worts? Hier wird auf \"nicht-WORD_CHAR gefolgt von WORD_CHAR\" geprüft, laut (etwas unklarer?!) Beschreibung bzw. Regexp im Originalcode wird aber gesucht: Wenn current ein Buchstabe ist, dann soll next keins sein; wenn current kein Buchstabe ist, dann soll next einer sein. Also (currentCharType == TextUtilities.WORD_CHAR) XOR (nextCharType == TextUtilities.WORD_CHAR)", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;112;Was genau ist ein WORD_CHAR: Ein Wort-Trennzeichen, oder Zeichen innerhalb eines Worts? Hier wird auf \"nicht-WORD_CHAR gefolgt von WORD_CHAR\" geprüft, laut (etwas unklarer?!) Beschreibung bzw. Regexp im Originalcode wird aber eine Grenze gesucht, wo ein", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;112;Was ist WORD_CHAR? Überprüft diese Methode genau das Gegenteil von dem, was sie überprüfen soll? Das Ende eines Wortes ist doch sicher WORD_CHAR gefolgt von nicht-WORD_CHAR; hier wird auf nicht-WORD_CHAR gefolgt von WORD_CHAR geprüft. Oder sind WORD_CHAR die Trennzeichen? Dann bitte entsprechend umbenennen.", FP.FP);
        putDef(ret, "#rR;112;if not closed", FP.FP); //code style
        putDef(ret, "#rR;112;it should be equal to the word_char", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;112;The control of \"current\" and \"next\" must be in \"and\"", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as false positive
        putDef(ret, "#rR;112;Curly braces", FP.FP); //code style
        putDef(ret, "#rR;112;Definition might be a bit too loose, unless anything that's not a WORD_CHAR is a line-break, symbol or space.", FP.FP);
        putDef(ret, "#rR;112;WORD_CHAR probably not contains digit which are not excluded by the requirements.", FP.FP);
        putDef(ret, "#rR;113;shouldn't this be return false?", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as a false positive
        putDef(ret, "#rR;113;false", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as a false positive
        putDef(ret, "#rR;113;false?", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as a false positive
        putDef(ret, "#rR;113;should return false", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as a false positive
        putDef(ret, "#rR;113;I’m not sure what this function is supposed to return in this case but I would bet on `false`", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;113;m.E prüft die Methode, ob man vor einem Wort steht", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;115;Was passiert, wenn es kein nächstes symbol gibt?", FP.FP); //is handled by caller
        putDef(ret, "#rR;115;Where are line start/ends checked?", FP.FP);
        putDef(ret, "#rR;116;\"==\" is wrong here, \"!=\" must be used because if \"current\" is a word character, \"next\" must be a separator in order to recognize a word boundary", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;Maybe != instead of ==", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;result must be negated?", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;The control of \"current\" and \"next\" must be in \"and\"", P2.IS_END_WORD_WRONG_RESULT); //could also be counted as false positive
        putDef(ret, "#rR;116;Next char should be symbol or space not wordchar", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;Isn't it != instead of == ?", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;Next char should be symbol or space not wordchar. So if it is wordchar --> false else true. This is not correct", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;If character is last in String (end of file without \n at the end of last file) will result in Exception", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;the method must return false if the next charType is a word character", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;this should be !=", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;should be an unequal comparison", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;Shouldn’t it be `!=`?", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;comparison for space is missing", P2.IS_END_WORD_WRONG_RESULT); //with some goodwill
        putDef(ret, "#rR;116;If character is last in String (end of file without \\n at the end of last file) will result in Exception", FP.FP); //is wrong
        putDef(ret, "#rR;116;should be != as well", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;check negation", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;This returns true when both chars are WORD_CHARs which is most probably a bug", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;!", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;If the next character is a word character, then the current character is not the end of a word. It is in the middle of a word?", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;Sowohl Symbole als auch Leerzeichen sind zulässig, oder auch EOL", P2.IS_END_WORD_WRONG_RESULT); //did the reviewer really mean this issue?
        putDef(ret, "#rR;116;Sowohl Symbole als auch Leerzeichen sind zulässig", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;inverted condition", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;!=", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;nextCharType should NOT be a WORD_CHAR.", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#rR;116;According to the definition in the commit message, shouldn' the two condition hold together? You return after checking only the first one and then check the second one here", P2.IS_END_WORD_WRONG_RESULT);
        putDef(ret, "#sR;120;Kommentare löschne", FP.FP); //documentation
        putDef(ret, "#sR;121;typo in comment", FP.FP); //documentation
        putDef(ret, "#sR;121;searches", FP.FP);
        putDef(ret, "#sR;121;search_es_", FP.FP);
        putDef(ret, "#sR;123;Rename to \"wholeWordSearcher\"", FP.FP); //variable naming
        putDef(ret, "#sR;123;rename to wholeWordsOnly", FP.FP); //variable naming
        putDef(ret, "#sR;123;is this ever used?", FP.FP); //yes it is and would not be defect anyway
        putDef(ret, "#sR;123;should be a private variable.", FP.FP); //code style
        putDef(ret, "#sR;123;should be private", FP.FP); //code style
        putDef(ret, "#tR;130;Class name is not equal to filename!", FP.FP); //is inner class
        putDef(ret, "#tR;131;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#tR;138;open braces might be come at end of previous line", FP.FP); //code style
        putDef(ret, "#tR;138;subsitutions are not printed", FP.FP); //we regard this as OK, matter of requirements
        putDef(ret, "#tR;139;Typo in String literal", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match*", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Orthographical mistake: It's match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo: Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match is misspelt.", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match not Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo : Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Mathc -> Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;check spelling", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Here is a typo int the String. It should be Match, right?", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo: Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo: Mathc->Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo?", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo: Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typin mistake should be Match not Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo in string", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo: \"Mathc\" instead of \"Match\"", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;this", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;should be Match instead of Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo \"Match\"", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo: Mathc --> Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;'Mathc' Misspelling in toString method", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo in Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo: match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match instead of Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Mathc -> Match; Substiturions wären vielleicht sinnvoll in Ausgabe", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;A typo", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo match", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Math-C? Match?", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Match (typo)=", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;typo : Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo, should read match.", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo: Match. Warum sind die substitutions nicht nennenswert?", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Rückgabewert Match statt Mathc?", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo in Mathc", P2.TYPO_MATCH);
        putDef(ret, "#tR;139;Typo: Mathc should be Match", P2.TYPO_MATCH);
        putDef(ret, "#uR;125;contain", FP.FP); //typo in comment
        putDef(ret, "#uR;125;Orthography: This should contain the separator ...", FP.FP); //documentation
        putDef(ret, "#uR;125;Orthography: This should contain", FP.FP);
        putDef(ret, "#uR;125;contains -> contain; list -> string; word -> \"no word\"", FP.FP); //documentation
        putDef(ret, "#uR;125;remove should", FP.FP); //documentation
        putDef(ret, "#uR;125;Widerspricht der Variablenbenennung. Vereinheitlichen.", FP.FP);
        putDef(ret, "#uR;125;contain instead of contain", FP.FP); //documentation
        putDef(ret, "#uR;126;that are valid characters.", FP.FP); //documentation
        putDef(ret, "#uR;125;This should contain", FP.FP);
        putDef(ret, "#uR;126;noWordSep is shared between buffers, according to comments above", FP.FP); //no
        putDef(ret, "#uR;126;that should be considered valid characters.", FP.FP);
        putDef(ret, "#uR;129;Type does not match comment. Should be a collection type or treatment of multiple chars should be documented", FP.FP); //unclear comment
        putDef(ret, "#uR;129;If it should contain a list of chars, then make the attribute of type List<String>", FP.FP); //design issue
        putDef(ret, "#uR;129;This should be just public. And change the name to nWordSeparators", FP.FP);
        putDef(ret, "#uR;129;variable should be private", FP.FP); //design issue
        putDef(ret, "#uR;129;an actual List<String> would be better", FP.FP); //design issue
        putDef(ret, "#uR;129;should be private.", FP.FP); //code style
        putDef(ret, "#uR;129;Javadoc and naming of the variable dosn't match", FP.FP); //documentation
        putDef(ret, "#uR;126;the comment contradicts how it is used.", FP.FP); //documentation

        putDef(ret, "#dR;232;If you call this method twice having a HyperSearchOperationNode, you may have race conditions on the noWordSep property. It is possible the your noWordSep property is resetted after another call of getSearchMatcher", FP.FP);
        putDef(ret, "#jR;146;beim Newline wird nicht die Position korrigiert", FP.FP);
        putDef(ret, "#jR;147;newline ist hier enthalten", FP.FP);
        putDef(ret, "#mR;89;shouldnt we get the start of the text in case of reverse == true?", FP.FP);
        putDef(ret, "#rR;115;Zeilenende wird nicht betrachtet", FP.FP);
        putDef(ret, "#qR;88;index is out-of-bounds if line is empty and an empty range is passed with start==end==0; this may be correct if the line ending is saved in the character sequence and is guaranteed to never be selected by the interval passed", FP.FP);
        putDef(ret, "#qR;90;isEndChar macht mit dem ersten und vorigen Symbol keinen Sinn.", FP.FP); //naming
        putDef(ret, "#qR;90;missing check for newline", FP.FP);
        putDef(ret, "#qR;96;index is out-of-bounds if end==length of line (i.e. the \"end\" character is the last character in the line); this may be correct if the line ending is saved in the character sequence and is guaranteed to never be selected by the interval passed", FP.FP);
        putDef(ret, "#qR;102;only the transitions at the start at end characters are checked, but there may be more within the character sequence, so the sequence may consist of multiple words which does not conform to the specification", FP.FP);
        putDef(ret, "#fR;655;unchecked cast", FP.FP);
        putDef(ret, "#nR;244;unchecked cast", FP.FP);
        putDef(ret, "#gR;1195;unchecked cast", FP.FP);
        return ret;
    }
}