library(lme4)
library(plyr)
library(dgof)
library(exact2x2)
library(ggplot2)
library(MASS)
library(Kendall)
library(effsize)

results <- read.table(
                    file = "results.csv", 
                    sep = ';', 
                    header = T,
                    na.strings = c("NA", "-1"))

results$X <- NULL
results$timeForReviewTotal <- results$netTimeForReviewWithIdlingR0 + results$netTimeForReviewWithIdlingR1 + results$netTimeForReviewWithIdlingR2
results$totalDurationMinutes <- results$totalDuration / 1000 / 60
results$timeForReviewMinutesR0 <- results$netTimeForReviewWithIdlingR0 / 1000 / 60
results$timeForReviewMinutesR1 <- results$netTimeForReviewWithIdlingR1 / 1000 / 60
results$timeForReviewMinutesR2 <- results$netTimeForReviewWithIdlingR2 / 1000 / 60
results$timeForReviewMinutesTotal <- results$timeForReviewTotal / 1000 / 60
results$timeForUnderstandingMinutesR0 <- results$timeForUnderstandingR0 / 1000 / 60
results$timeForUnderstandingMinutesR1 <- results$timeForUnderstandingR1 / 1000 / 60
results$timeForUnderstandingMinutesR2 <- results$timeForUnderstandingR2 / 1000 / 60
results$correctnessUnderstandingR0 <- results$understandingCorrectR0 / results$understandingTotalR0
results$correctnessUnderstandingR1 <- results$understandingCorrectR1 / results$understandingTotalR1
results$correctnessUnderstandingR2 <- results$understandingCorrectR2 / results$understandingTotalR2
results$efficiencyR0 <- results$defectsFoundR0 / results$timeForReviewMinutesR0 * 60
results$efficiencyR1 <- results$defectsFoundR1 / results$timeForReviewMinutesR1 * 60
results$efficiencyR2 <- results$defectsFoundR2 / results$timeForReviewMinutesR2 * 60
results$efficiencyR0P1 <- (results$defectsFoundR0 + 1) / results$timeForReviewMinutesR0 * 60
results$efficiencyTotal <- (results$defectsFoundR0 + results$defectsFoundR1 + results$defectsFoundR2) / results$timeForReviewMinutesTotal * 60
results$effectivenessR0 <- results$defectsFoundR0 / results$defectsTotalR0
results$effectivenessR1 <- results$defectsFoundR1 / results$defectsTotalR1
results$effectivenessR2 <- results$defectsFoundR2 / results$defectsTotalR2
results$defectsFoundTotal <- results$defectsFoundR0 + results$defectsFoundR1 + results$defectsFoundR2
results$defectsTotalTotal <- results$defectsTotalR0 + results$defectsTotalR1 + results$defectsTotalR2
results$effectivenessTotal <- results$defectsFoundTotal / results$defectsTotalTotal
results$betterEffiIsDiff1 <- ifelse(results$efficiencyR1 > results$efficiencyR2, results$changeR1 == "P1", results$changeR2 == "P1")
results$betterEffiIsR2 <- results$efficiencyR1 < results$efficiencyR2
results$betterEffiIsBetterTreatment <- ifelse(
          results$efficiencyR1 > results$efficiencyR2, 
          results$treatmentR1 %in% c("OF", "ONF") | results$treatmentR2 == "WNF", 
          results$treatmentR2 %in% c("OF", "ONF") | results$treatmentR1 == "WNF")
results$betterTreatmentIsR1 <- results$treatmentR1 %in% c("OF", "ONF") | results$treatmentR2 == "WNF"
results$efficiencyOfBetterTreatment <- ifelse(results$betterTreatmentIsR1, results$efficiencyR1, results$efficiencyR2)
results$efficiencyOfWorseTreatment <- ifelse(results$betterTreatmentIsR1, results$efficiencyR2, results$efficiencyR1)
results$effiDifference <- results$efficiencyOfBetterTreatment - results$efficiencyOfWorseTreatment
results$delocalisedDefectsFoundWU <- results$d_WU_FROM_TO_SWAP
results$delocalisedDefectsFoundD1 <- results$d_D1_FROM_TO_SWITCHED_RENAME1 + results$d_D1_FROM_TO_SWITCHED_RENAME2
results$delocalisedDefectsFoundD2 <- results$d_D2_MATCH_END_NOT_SET + results$d_D2_WHOLEWORDMISSING + results$d_D2_WRONG_PROPERTY
results$delocalisedDefectsFoundTotal <- results$delocalisedDefectsFoundD1 + results$delocalisedDefectsFoundD2 + results$delocalisedDefectsFoundWU
results$delocalisedDefectsTotal <- 6
results$localisedDefectsFoundTotal <- results$defectsFoundTotal - results$delocalisedDefectsFoundTotal
results$lowWmspan <- results$wmspanScore < median(results$wmspanScore, na.rm=T)
results$understandingForBetterTreatment <- ifelse(results$betterTreatmentIsR1, results$understandingCorrectR1 / results$understandingTotalR1, results$understandingCorrectR2 / results$understandingTotalR2)
results$understandingForWorseTreatment <- ifelse(results$betterTreatmentIsR1, results$understandingCorrectR2 / results$understandingTotalR2, results$understandingCorrectR1 / results$understandingTotalR1)

#remove participants that participated multiple times
results <- results[results$excludeAsDuplicate == "false",]

dropoutR1 <- results[!is.na(results$defectsFoundR0) & is.na(results$defectsFoundR1) & is.na(results$defectsFoundR2),]
dropoutR2 <- results[!is.na(results$defectsFoundR0) & !is.na(results$defectsFoundR1) & is.na(results$defectsFoundR2),]
dropoutWM <- results[is.na(results$wmspanScore) & !is.na(results$defectsFoundR2),]

#remove all records that did not complete all reviews. rationale: we don't know when during the reviews they lost interest
fullData <- results
results <- results[!(is.na(results$defectsFoundR0) | is.na(results$defectsFoundR1) | is.na(results$defectsFoundR2)),]

resultsOnlyWm <- results[!is.na(results$wmspanScore),]



####
# check some assertions
####
stopifnot(results$defectsFoundAfterTenMinR0 <= results$defectsTotalR0)
stopifnot(results$defectsFoundAfterTwentyMinR0 <= results$defectsTotalR0)
stopifnot(results$defectsFoundAfterThirtyMinR0 <= results$defectsTotalR0)
stopifnot(results$defectsFoundR0Total <= results$defectsTotalR0)
stopifnot(results$defectsFoundAfterTenMinR1 <= results$defectsTotalR1)
stopifnot(results$defectsFoundAfterTwentyMinR1 <= results$defectsTotalR1)
stopifnot(results$defectsFoundAfterThirtyMinR1 <= results$defectsTotalR1)
stopifnot(results$defectsFoundR1Total <= results$defectsTotalR1)
stopifnot(results$defectsFoundAfterTenMinR2 <= results$defectsTotalR2)
stopifnot(results$defectsFoundAfterTwentyMinR2 <= results$defectsTotalR2)
stopifnot(results$defectsFoundAfterThirtyMinR2 <= results$defectsTotalR2)
stopifnot(results$defectsFoundR2Total <= results$defectsTotalR2)
stopifnot(is.na(results$wmspanScore) | results$wmspanScore <= 50)
stopifnot(results$netTimeForReviewWithoutIdlingR0 <= results$netTimeForReviewWithIdlingR0)
stopifnot(results$netTimeForReviewWithIdlingR0 <= results$grossTimeForReviewR0)
stopifnot(results$netTimeForReviewWithoutIdlingR1 <= results$netTimeForReviewWithIdlingR1)
stopifnot(results$netTimeForReviewWithIdlingR1 <= results$grossTimeForReviewR1)
stopifnot(results$netTimeForReviewWithoutIdlingR2 <= results$netTimeForReviewWithIdlingR2)
stopifnot(results$netTimeForReviewWithIdlingR2 <= results$grossTimeForReviewR2)

####
# some functions need the data in long format, so convert it
####

toLong <- function(resultFrame) {
  res <- reshape(resultFrame, 
                         varying = c(
                           "treatmentR1", 
                           "changeR1", 
                           "netTimeForReviewWithIdlingR1", 
                           "netTimeForReviewWithoutIdlingR1", 
                           "grossTimeForReviewR1", 
                           "timeForReviewMinutesR1", 
                           "defectsFoundR1", 
                           "defectsFoundAfterTenMinR1",
                           "defectsFoundAfterTwentyMinR1",
                           "defectsFoundAfterThirtyMinR1",
                           "defectsTotalR1", 
                           "timeForUnderstandingR1", 
                           "timeForUnderstandingMinutesR1", 
                           "understandingCorrectR1", 
                           "understandingTotalR1", 
                           "correctnessUnderstandingR1",
                           "efficiencyR1",
                           "effectivenessR1",
                           "scrollAmountR1",
                           "pauseCountR1",
                           "descriptionCountR1",
                           "treatmentR2", 
                           "changeR2", 
                           "netTimeForReviewWithIdlingR2", 
                           "netTimeForReviewWithoutIdlingR2", 
                           "grossTimeForReviewR2", 
                           "timeForReviewMinutesR2", 
                           "defectsFoundR2", 
                           "defectsFoundAfterTenMinR2",
                           "defectsFoundAfterTwentyMinR2",
                           "defectsFoundAfterThirtyMinR2",
                           "defectsTotalR2", 
                           "timeForUnderstandingR2", 
                           "timeForUnderstandingMinutesR2", 
                           "understandingCorrectR2", 
                           "understandingTotalR2",
                           "correctnessUnderstandingR2",
                           "efficiencyR2",
                           "effectivenessR2",
                           "scrollAmountR2",
                           "pauseCountR2",
                           "descriptionCountR2"), 
                         direction = "long", 
                         timevar = "reviewNumber",
                         sep = "")
  return(res)
}
resultsLong <- toLong(results)
resultsLong$treatmentONF <- as.numeric(resultsLong$treatmentR == "ONF");
resultsLong$treatmentOF <- as.numeric(resultsLong$treatmentR == "OF");
resultsLong$treatmentWNF <- as.numeric(resultsLong$treatmentR == "WNF");
resultsLong$treatmentWF <- as.numeric(resultsLong$treatmentR == "WF");

#####
# start printing the data for latex
#####

writeLatexValue <- function(f, name, value) {
  writeLatexValueExact(f, name, round(value, digits=2))
}

writeLatexValue1E <- function(f, name, value) {
  writeLatexValueExact(f, name, round(value, digits=1), minDigits=1)
}

writeLatexValueInt <- function(f, name, value) {
  writeLatexValueExact(f, name, round(value, digits=0))
}

writeLatexValue4 <- function(f, name, value) {
  writeLatexValueExact(f, name, round(value, digits=4))
}

writeLatexValue4E <- function(f, name, value) {
  writeLatexValueExact(f, name, round(value, digits=4), minDigits=4)
}

writeLatexValueExact <- function(f, name, value, minDigits=0) {
  writeLatexValueStr(f, name, format(value, scientific=F, nsmall = minDigits))
}

writeLatexValueStr <- function(f, name, value) {
  cat("\\newcommand{\\dta", name, "}{", value, "}\n", file=f, sep="")
}

outputFile <- file("data.tex", "w")

# basic descriptive statistics

writeLatexValue(outputFile, "ParticipantCountWithIncompletes", length(fullData$pId))
writeLatexValue(outputFile, "ParticipantCount", length(results$pId))
writeLatexValue(outputFile, "WmspanCount", length(results[!is.na(results$wmspanScore),]$pId))
writeLatexValue(outputFile, "ProgrammerCount", length(results[results$role=="programmer",]$pId))
writeLatexValue(outputFile, "ResearcherCount", length(results[results$role=="researcher",]$pId))
writeLatexValue(outputFile, "ProgramAtLeastWeekly", length(results[results$programPract >= 3,]$pId))
writeLatexValue(outputFile, "ReviewAtLeastWeekly", length(results[results$reviewPract >= 3,]$pId))
writeLatexValue(outputFile, "AdvancedDevelopers", length(results[results$profDevExp >= 3,]$pId))
writeLatexValue(outputFile, "NoExpDevelopers", length(results[results$profDevExp == 0,]$pId))
writeLatexValue(outputFile, "WmspanScoreMin", min(resultsOnlyWm$wmspanScore))
writeLatexValue(outputFile, "WmspanScoreMax", max(resultsOnlyWm$wmspanScore))
writeLatexValue(outputFile, "WmspanScoreMean", mean(resultsOnlyWm$wmspanScore))
writeLatexValue(outputFile, "WmspanScoreSd", sd(resultsOnlyWm$wmspanScore))
writeLatexValue(outputFile, "WmspanScoreMedian", median(resultsOnlyWm$wmspanScore))

tblExpCont <- count(results[results$controlledSetting == "true",]$profDevExp)
names(tblExpCont) <- c("experience", "count controlled")
tblExpOnl <- count(results[results$controlledSetting == "false",]$profDevExp)
names(tblExpOnl) <- c("experience", "count online")
tblExpMerged <- merge(tblExpCont, tblExpOnl, all=T)
tblExpMerged[is.na(tblExpMerged)] <- 0
pdf("participantExperience.pdf", width=7, height=5)
par(mar=c(2.3,4.0,0.1,0.1), cex=1.5)
barplot(t(as.matrix(tblExpMerged[,c("count controlled", "count online")])),
        names.arg = c("no", "1 year", "2 yrs", "3 - 5", "6 - 10", "more"),
        ylab="participant count")
dev.off()

groupDistribution <- table(paste(results$treatmentCombination, " (", results$treatmentR1, "+", results$changeR1, ")", sep=""))
write.table(groupDistribution, file="groupDistribution.csv", quote=FALSE, col.names = c("Var1", "Number of participants"), row.names=F, sep=",")

writeLatexValue(outputFile, "MeanDefectsFoundRZ", mean(results$defectsFoundR0))
writeLatexValue(outputFile, "MeanDefectsFoundRO", mean(resultsLong[resultsLong$reviewNumber == 1,]$defectsFoundR))
writeLatexValue(outputFile, "MeanDefectsFoundRT", mean(resultsLong[resultsLong$reviewNumber == 2,]$defectsFoundR))
writeLatexValue(outputFile, "MeanDefectsFoundPO", mean(resultsLong[resultsLong$changeR == "P1",]$defectsFoundR))
writeLatexValue(outputFile, "MeanDefectsFoundPT", mean(resultsLong[resultsLong$changeR == "P2",]$defectsFoundR))
writeLatexValue(outputFile, "TotalDefectsPZ", mean(results$defectsTotalR0))
writeLatexValue(outputFile, "TotalDefectsPO", mean(resultsLong[resultsLong$changeR == "P1",]$defectsTotalR))
writeLatexValue(outputFile, "TotalDefectsPT", mean(resultsLong[resultsLong$changeR == "P2",]$defectsTotalR))
writeLatexValue(outputFile, "MeanDelDefectsFoundRZ", mean(results$delocalisedDefectsFoundWU))
writeLatexValue(outputFile, "MeanDelDefectsFoundRO", mean(ifelse(results$changeR1 == "P1", results$delocalisedDefectsFoundD1, results$delocalisedDefectsFoundD2)))
writeLatexValue(outputFile, "MeanDelDefectsFoundRT", mean(ifelse(results$changeR1 == "P1", results$delocalisedDefectsFoundD2, results$delocalisedDefectsFoundD1)))
writeLatexValue(outputFile, "MeanDelDefectsFoundPO", mean(results$delocalisedDefectsFoundD1))
writeLatexValue(outputFile, "MeanDelDefectsFoundPT", mean(results$delocalisedDefectsFoundD2))
writeLatexValue(outputFile, "TotalDelDefectsPZ", 1)
writeLatexValue(outputFile, "TotalDelDefectsPO", 2)
writeLatexValue(outputFile, "TotalDelDefectsPT", 3)
writeLatexValue(outputFile, "MeanDurationRZ", mean(results$timeForReviewMinutesR0))
writeLatexValue(outputFile, "MeanDurationRO", mean(resultsLong[resultsLong$reviewNumber == 1,]$timeForReviewMinutesR))
writeLatexValue(outputFile, "MeanDurationRT", mean(resultsLong[resultsLong$reviewNumber == 2,]$timeForReviewMinutesR))
writeLatexValue(outputFile, "MeanDurationPO", mean(resultsLong[resultsLong$changeR == "P1",]$timeForReviewMinutesR))
writeLatexValue(outputFile, "MeanDurationPT", mean(resultsLong[resultsLong$changeR == "P2",]$timeForReviewMinutesR))

writeLatexValue(outputFile, "SdDefectsFoundRZ", sd(results$defectsFoundR0))
writeLatexValue(outputFile, "SdDefectsFoundRO", sd(resultsLong[resultsLong$reviewNumber == 1,]$defectsFoundR))
writeLatexValue(outputFile, "SdDefectsFoundRT", sd(resultsLong[resultsLong$reviewNumber == 2,]$defectsFoundR))
writeLatexValue(outputFile, "SdDefectsFoundPO", sd(resultsLong[resultsLong$changeR == "P1",]$defectsFoundR))
writeLatexValue(outputFile, "SdDefectsFoundPT", sd(resultsLong[resultsLong$changeR == "P2",]$defectsFoundR))
writeLatexValue(outputFile, "SdDelDefectsFoundRZ", sd(results$delocalisedDefectsFoundWU))
writeLatexValue(outputFile, "SdDelDefectsFoundRO", sd(ifelse(results$changeR1 == "P1", results$delocalisedDefectsFoundD1, results$delocalisedDefectsFoundD2)))
writeLatexValue(outputFile, "SdDelDefectsFoundRT", sd(ifelse(results$changeR1 == "P1", results$delocalisedDefectsFoundD2, results$delocalisedDefectsFoundD1)))
writeLatexValue(outputFile, "SdDelDefectsFoundPO", sd(results$delocalisedDefectsFoundD1))
writeLatexValue(outputFile, "SdDelDefectsFoundPT", sd(results$delocalisedDefectsFoundD2))
writeLatexValue(outputFile, "SdDurationRZ", sd(results$timeForReviewMinutesR0))
writeLatexValue(outputFile, "SdDurationRO", sd(resultsLong[resultsLong$reviewNumber == 1,]$timeForReviewMinutesR))
writeLatexValue(outputFile, "SdDurationRT", sd(resultsLong[resultsLong$reviewNumber == 2,]$timeForReviewMinutesR))
writeLatexValue(outputFile, "SdDurationPO", sd(resultsLong[resultsLong$changeR == "P1",]$timeForReviewMinutesR))
writeLatexValue(outputFile, "SdDurationPT", sd(resultsLong[resultsLong$changeR == "P2",]$timeForReviewMinutesR))

writeLatexValue(outputFile, "DescWuEffOFWF", mean(results[results$treatmentCombination == "OF-WF",]$efficiencyR0))
writeLatexValue(outputFile, "DescWuEffONFWNF", mean(results[results$treatmentCombination == "ONF-WNF",]$efficiencyR0))
writeLatexValue(outputFile, "DescWuEffWFWNF", mean(results[results$treatmentCombination == "WF-WNF",]$efficiencyR0))

writeLatexValue1E(outputFile, "MeanEffiCompany", mean(results[results$controlledSetting == "true",]$efficiencyTotal))
writeLatexValue1E(outputFile, "MeanEffiOnline", mean(results[results$controlledSetting == "false",]$efficiencyTotal))
writeLatexValueInt(outputFile, "MeanEffeCompany", mean(results[results$controlledSetting == "true",]$effectivenessTotal) * 100)
writeLatexValueInt(outputFile, "MeanEffeOnline", mean(results[results$controlledSetting == "false",]$effectivenessTotal) * 100)

writeLatexValue(outputFile, "PauseUserCount", length(results[results$pauseCountR0 + results$pauseCountR1 + results$pauseCountR2 > 0,]$pId))

# RQ 1

ggplot(resultsOnlyWm, aes(wmspanScore, delocalisedDefectsFoundTotal)) + 
  geom_jitter(width = 0.1, height = 0.1, shape = 1) + 
  xlab("working memory span test score") + 
  ylab("# total delocalized defects found") + 
  ylim(-0.1, 15.1) +
  theme_bw() +
  ggsave("scatterplotWmspanDelocalized.pdf", width=3, height=3)

ggplot(resultsOnlyWm, aes(wmspanScore, localisedDefectsFoundTotal)) + 
  geom_jitter(width = 0.1, height = 0.1, shape = 2) + 
  xlab("working memory span test score") + 
  ylab("# total other defects found") + 
  ylim(-0.1, 15.1) +
  theme_bw() +
  ggsave("scatterplotWmspanLocalized.pdf", width=3, height=3)

#two-tailed tests, p-value will have to be divided by two
kendallResultAll <- Kendall(resultsOnlyWm$wmspanScore, resultsOnlyWm$defectsFoundTotal)
kendallResultDel <- Kendall(resultsOnlyWm$wmspanScore, resultsOnlyWm$delocalisedDefectsFoundTotal)
kendallResultLoc <- Kendall(resultsOnlyWm$wmspanScore, resultsOnlyWm$localisedDefectsFoundTotal)

kendallForSubset <- function(leaveOut) {
  curSubset <- resultsOnlyWm[resultsOnlyWm$pId != leaveOut,]
  kendallResultDelSubset <- Kendall(curSubset$wmspanScore, curSubset$delocalisedDefectsFoundTotal)
  return(data.frame(leaveOut, kendallResultDelSubset$tau, kendallResultDelSubset$sl / 2))
}
influentialDataPointTable <- do.call(rbind, lapply(resultsOnlyWm$pId, kendallForSubset))

#there is a warning for the kendall p-value due to the ties, therefore check whether random tie breaking leads to different results
jitteredTest <- function(dummy) {return(Kendall(jitter(resultsOnlyWm$wmspanScore, amount=0.1), jitter(resultsOnlyWm$delocalisedDefectsFoundTotal, amount=0.1))$sl)}
stopifnot(abs(median(sapply(1:1000, jitteredTest)) - kendallResultDel$sl) < 0.005)


writeLatexValue(outputFile, "RQOkendallTauAll", kendallResultAll$tau)
writeLatexValue4(outputFile, "RQOkendallPvalueAll", kendallResultAll$sl / 2)
writeLatexValue(outputFile, "RQOkendallTauDel", kendallResultDel$tau)
writeLatexValue4(outputFile, "RQOkendallPvalueDel", kendallResultDel$sl / 2)
writeLatexValue(outputFile, "RQOkendallTauLoc", kendallResultLoc$tau)

varsForCorrelTable <- list(
  "wmspanScore",
  "defectsFoundTotal",
  "delocalisedDefectsFoundTotal",
  "localisedDefectsFoundTotal",
  "timeForReviewMinutesTotal",
  "controlledSetting",
  "profDevExp",
  "reviewPract",
  "viewHeight",
  "programPract",
  "javaExp",
  "changeR1",
  "workStartHours",
  "fitness")

for (fA in varsForCorrelTable) {
  for (fB in varsForCorrelTable) {
    writeLatexValue(outputFile, 
                    paste0("RQOkendallTau", gsub("1", "O", fA), "VS", gsub("1", "O", fB)), 
                    Kendall(resultsOnlyWm[[fA]], resultsOnlyWm[[fB]])$tau)
  }
}



initialDefectModel <- lm(defectsFoundTotal ~ 1, resultsOnlyWm)
optimalDefectModel <- stepAIC(initialDefectModel, 
                           scope = list(
                             upper = ~ profDevExp + javaExp + programPract + reviewPract + workStartHours + fitness + jedit + prize + wmspanScore + viewWidth + viewHeight + browser + changeR1 + controlledSetting + timeForReviewMinutesTotal,
                             lower = ~ 1),
                           k=log(length(resultsOnlyWm$profDevExp)),
                           trace=F)
print(optimalDefectModel)
writeLatexValue4(outputFile, "RQOallDefModelIntercept", optimalDefectModel$coefficients["(Intercept)"])
writeLatexValue4(outputFile, "RQOallDefModelCoefTime", optimalDefectModel$coefficients["timeForReviewMinutesTotal"])
writeLatexValue4(outputFile, "RQOallDefModelRSq", summary(optimalDefectModel)$r.squared)
writeLatexValue4(outputFile, "RQOallDefModelARSq", summary(optimalDefectModel)$adj.r.squared)
writeLatexValue(outputFile, "RQOallDefModelBIC", BIC(optimalDefectModel))

initialDelDefectModel <- lm(delocalisedDefectsFoundTotal ~ 1, resultsOnlyWm)
optimalDelDefectModel <- stepAIC(initialDelDefectModel, 
                              scope = list(
                                upper = ~ profDevExp + javaExp + programPract + reviewPract + workStartHours + fitness + jedit + prize + wmspanScore + viewWidth + viewHeight + browser + changeR1 + controlledSetting + timeForReviewMinutesTotal,
                                lower = ~ 1),
                              k=log(length(resultsOnlyWm$profDevExp)),
                              trace=F)
print(optimalDelDefectModel)
writeLatexValue4(outputFile, "RQOdelDefModelIntercept", optimalDelDefectModel$coefficients["(Intercept)"])
writeLatexValue4(outputFile, "RQOdelDefModelCoefTime", optimalDelDefectModel$coefficients["timeForReviewMinutesTotal"])
writeLatexValue4(outputFile, "RQOdelDefModelCoefWmspan", optimalDelDefectModel$coefficients["wmspanScore"])
writeLatexValue4(outputFile, "RQOdelDefModelCoefReviewPract", optimalDelDefectModel$coefficients["reviewPract"])
writeLatexValue4(outputFile, "RQOdelDefModelCoefContr", optimalDelDefectModel$coefficients["controlledSettingtrue"])
writeLatexValue4(outputFile, "RQOdelDefModelRSq", summary(optimalDelDefectModel)$r.squared)
writeLatexValue4(outputFile, "RQOdelDefModelARSq", summary(optimalDelDefectModel)$adj.r.squared)
writeLatexValue(outputFile, "RQOdelDefModelBIC", BIC(optimalDelDefectModel))


# RQ 2

writeLatexValueInt(outputFile, "RQTmeanEffeSmall", mean(results$effectivenessR0) * 100)
writeLatexValueInt(outputFile, "RQTmeanEffeLarge", mean(c(results$effectivenessR1, results$effectivenessR2)) * 100)
writeLatexValue(outputFile, "RQTmeanEffiSmall", mean(results$efficiencyR0))
writeLatexValue(outputFile, "RQTmeanEffiLarge", mean(c(results$efficiencyR1, results$efficiencyR2)))

countFoundWU <- sum(results$d_WU_FROM_TO_SWAP)
countNotFoundWU <- length(results$pId) - countFoundWU

countFoundR1WU <- sum(results[results$changeR1=="P1",]$d_WU_FROM_TO_SWAP)
countFoundR2WU <- sum(results[results$changeR1=="P2",]$d_WU_FROM_TO_SWAP)

countFoundR1S1 <- sum(results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME1)
countNotFoundR1S1 <- length(results[results$changeR1=="P1",]$pId) - countFoundR1S1
countFoundR2S1 <- sum(results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME1)
countNotFoundR2S1 <- length(results[results$changeR1=="P2",]$pId) - countFoundR2S1

countFoundR1S2 <- sum(results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME2)
countNotFoundR1S2 <- length(results[results$changeR1=="P1",]$pId) - countFoundR1S2
countFoundR2S2 <- sum(results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME2)
countNotFoundR2S2 <- length(results[results$changeR1=="P2",]$pId) - countFoundR2S2

writeLatexValue(outputFile, "RQTfoundWU", countFoundWU)
writeLatexValue(outputFile, "RQTfoundROWU", countFoundR1WU)
writeLatexValue(outputFile, "RQTfoundRTWU", countFoundR2WU)
writeLatexValue(outputFile, "RQTfoundROSO", countFoundR1S1)
writeLatexValue(outputFile, "RQTfoundRTSO", countFoundR2S1)
writeLatexValue(outputFile, "RQTfoundROST", countFoundR1S2)
writeLatexValue(outputFile, "RQTfoundRTST", countFoundR2S2)
writeLatexValue(outputFile, "RQTtotalWU", countFoundWU + countNotFoundWU)
writeLatexValue(outputFile, "RQTtotalRO", countFoundR1S1 + countNotFoundR1S1)
writeLatexValue(outputFile, "RQTtotalRT", countFoundR2S1 + countNotFoundR2S1)

writeRQTdetailCounts <- function(prefix, patchInFirst, defectVector) {
  writeLatexValue(outputFile, paste(prefix, "none", sep=""), length(
    results[results$changeR1==patchInFirst & results$d_WU_FROM_TO_SWAP == 0 & defectVector == 0,]$shortId))
  writeLatexValue(outputFile, paste(prefix, "small", sep=""), length(
    results[results$changeR1==patchInFirst & results$d_WU_FROM_TO_SWAP == 1 & defectVector == 0,]$shortId))
  writeLatexValue(outputFile, paste(prefix, "large", sep=""), length(
    results[results$changeR1==patchInFirst & results$d_WU_FROM_TO_SWAP == 0 & defectVector == 1,]$shortId))
  writeLatexValue(outputFile, paste(prefix, "both", sep=""), length(
    results[results$changeR1==patchInFirst & results$d_WU_FROM_TO_SWAP == 1 & defectVector == 1,]$shortId))
}
writeRQTdetailCounts("RQTcountROSO", "P1", results$d_D1_FROM_TO_SWITCHED_RENAME1)
writeRQTdetailCounts("RQTcountROST", "P1", results$d_D1_FROM_TO_SWITCHED_RENAME2)
writeRQTdetailCounts("RQTcountRTSO", "P2", results$d_D1_FROM_TO_SWITCHED_RENAME1)
writeRQTdetailCounts("RQTcountRTST", "P2", results$d_D1_FROM_TO_SWITCHED_RENAME2)

mcnemar.onesided <- function(x, y) { 
  cnt1 <- sum(x & !y);
  cnt2 <- sum(!x & y);
  return(pbinom(q=cnt2, size=(cnt1+cnt2), prob=.5)) 
}

cohen.g <- function(x, y) { 
  b <- sum(x & !y);
  c <- sum(!x & y);
  n <- b + c;
  P <- max(b / n, c / n);
  return(P - 0.5) 
}

writeLatexValue4(outputFile, "RQTMcNemarROSO", mcnemar.onesided(results[results$changeR1=="P1",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME1 == 1))
writeLatexValue4(outputFile, "RQTMcNemarROST", mcnemar.onesided(results[results$changeR1=="P1",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME2 == 1))
writeLatexValue4(outputFile, "RQTMcNemarRTSO", mcnemar.onesided(results[results$changeR1=="P2",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME1 == 1))
writeLatexValue4(outputFile, "RQTMcNemarRTST", mcnemar.onesided(results[results$changeR1=="P2",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME2 == 1))

writeLatexValue(outputFile, "RQTCohenROSO", cohen.g(results[results$changeR1=="P1",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME1 == 1))
writeLatexValue(outputFile, "RQTCohenROST", cohen.g(results[results$changeR1=="P1",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P1",]$d_D1_FROM_TO_SWITCHED_RENAME2 == 1))
writeLatexValue(outputFile, "RQTCohenRTSO", cohen.g(results[results$changeR1=="P2",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME1 == 1))
writeLatexValue(outputFile, "RQTCohenRTST", cohen.g(results[results$changeR1=="P2",]$d_WU_FROM_TO_SWAP == 1, results[results$changeR1=="P2",]$d_D1_FROM_TO_SWITCHED_RENAME2 == 1))

resultsOnlyWm$lossS1 <- resultsOnlyWm$d_WU_FROM_TO_SWAP == 1 & resultsOnlyWm$d_D1_FROM_TO_SWITCHED_RENAME1 == 0
resultsOnlyWm$lossS2 <- resultsOnlyWm$d_WU_FROM_TO_SWAP == 1 & resultsOnlyWm$d_D1_FROM_TO_SWITCHED_RENAME2 == 0
fisher.test(matrix(c(length(resultsOnlyWm[resultsOnlyWm$lowWmspan & resultsOnlyWm$lossS1,]$pId), 
                     length(resultsOnlyWm[!resultsOnlyWm$lowWmspan & resultsOnlyWm$lossS1,]$pId), 
                     length(resultsOnlyWm[resultsOnlyWm$lowWmspan & !resultsOnlyWm$lossS1 & resultsOnlyWm$d_WU_FROM_TO_SWAP == 1,]$pId), 
                     length(resultsOnlyWm[!resultsOnlyWm$lowWmspan & !resultsOnlyWm$lossS1 & resultsOnlyWm$d_WU_FROM_TO_SWAP == 1,]$pId)),
                   dimnames = list(c("loss", "no loss"), c("low", "high")),
                   nrow=2))
fisher.test(matrix(c(length(resultsOnlyWm[resultsOnlyWm$lowWmspan & resultsOnlyWm$lossS2,]$pId), 
                     length(resultsOnlyWm[!resultsOnlyWm$lowWmspan & resultsOnlyWm$lossS2,]$pId), 
                     length(resultsOnlyWm[resultsOnlyWm$lowWmspan & !resultsOnlyWm$lossS2 & resultsOnlyWm$d_WU_FROM_TO_SWAP == 1,]$pId), 
                     length(resultsOnlyWm[!resultsOnlyWm$lowWmspan & !resultsOnlyWm$lossS2 & resultsOnlyWm$d_WU_FROM_TO_SWAP == 1,]$pId)), 
                   dimnames = list(c("loss", "no loss"), c("low", "high")),
                   nrow=2))

# RQ 3

pdf("boxplotsOrders.pdf")
par(mfrow=c(3,3), mar=c(2.5,4.0,0.1,0.1))
boxplot(efficiencyR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "OF-WF",]), ylim=c(0, max(resultsLong$efficiencyR)), ylab="Efficiency")
boxplot(efficiencyR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "ONF-WNF",]), ylim=c(0, max(resultsLong$efficiencyR)))
boxplot(efficiencyR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "WF-WNF",]), ylim=c(0, max(resultsLong$efficiencyR)))
boxplot(effectivenessR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "OF-WF",]), ylim=c(0, max(resultsLong$effectivenessR)), ylab="Effectiveness")
boxplot(effectivenessR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "ONF-WNF",]), ylim=c(0, max(resultsLong$effectivenessR)))
boxplot(effectivenessR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "WF-WNF",]), ylim=c(0, max(resultsLong$effectivenessR)))
boxplot(timeForReviewMinutesR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "OF-WF",]), ylim=c(0, max(resultsLong$timeForReviewMinutesR)), ylab="Review Time")
boxplot(timeForReviewMinutesR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "ONF-WNF",]), ylim=c(0, max(resultsLong$timeForReviewMinutesR)))
boxplot(timeForReviewMinutesR ~ treatmentR, data = droplevels(resultsLong[resultsLong$treatmentCombination == "WF-WNF",]), ylim=c(0, max(resultsLong$timeForReviewMinutesR)))
dev.off()

createStats <- function(file, v, prefix) {
  ci <- sort(v)[qbinom(c(.025,.975), length(v), 0.5)]
  writeLatexValue1E(file, paste(prefix, "Median", sep=""), median(v))
  writeLatexValue1E(file, paste(prefix, "MedianCiLow", sep=""), ci[1])
  writeLatexValue1E(file, paste(prefix, "MedianCiHigh", sep=""), ci[2])
  writeLatexValue1E(file, paste(prefix, "Min", sep=""), min(v))
  writeLatexValue1E(file, paste(prefix, "Max", sep=""), max(v))
  writeLatexValue1E(file, paste(prefix, "Mean", sep=""), mean(v))
  writeLatexValue1E(file, paste(prefix, "Sd", sep=""), sd(v))
}

createRQHrow <- function(file, subframe, id) {
  createStats(file, subframe$efficiencyOfBetterTreatment, paste(id, "Better", sep=""))
  createStats(file, subframe$efficiencyOfWorseTreatment, paste(id, "Worse", sep=""))
  createStats(file, subframe$understandingForBetterTreatment * 100, paste(id, "UnderstandingBetter", sep=""))
  createStats(file, subframe$understandingForWorseTreatment * 100, paste(id, "UnderstandingWorse", sep=""))
  
  mb <- median(subframe$efficiencyOfBetterTreatment)
  mw <- median(subframe$efficiencyOfWorseTreatment)
  writeLatexValueInt(file, paste(id, "RelDiffMedians", sep=""), (mb - mw) / mw * 100)
  wtest <- wilcox.test(subframe$efficiencyOfBetterTreatment, subframe$efficiencyOfWorseTreatment, paired = T)
  writeLatexValue4E(file, paste(id, "WilcoxPvalue", sep=""), wtest$p.value)
  cliffResult <- cliff.delta(subframe$efficiencyOfBetterTreatment, subframe$efficiencyOfWorseTreatment)
  writeLatexValue(file, paste(id, "CliffD", sep=""), cliffResult$estimate)
  writeLatexValueStr(file, paste(id, "CliffDqual", sep=""), ifelse(cliffResult$magnitude == "negligible", "negl.", as.character(cliffResult$magnitude)))
  writeLatexValue(file, paste(id, "Count", sep=""), length(subframe$pId))
  writeLatexValue(file, paste(id, "Shapiro", sep=""), shapiro.test(subframe$efficiencyOfBetterTreatment - subframe$efficiencyOfWorseTreatment)$p.value)
}

createRQHrow(outputFile, results, "RQHOverallAll")
createRQHrow(outputFile, resultsOnlyWm[resultsOnlyWm$lowWmspan,], "RQHOverallLow")
createRQHrow(outputFile, results[results$treatmentCombination=="OF-WF",], "RQHOFWFAll")
createRQHrow(outputFile, results[results$treatmentCombination=="ONF-WNF",], "RQHONFWNFAll")
createRQHrow(outputFile, results[results$treatmentCombination=="WF-WNF",], "RQHWFWNFAll")
createRQHrow(outputFile, resultsOnlyWm[resultsOnlyWm$treatmentCombination=="OF-WF" & resultsOnlyWm$lowWmspan,], "RQHOFWFLow")
createRQHrow(outputFile, resultsOnlyWm[resultsOnlyWm$treatmentCombination=="ONF-WNF" & resultsOnlyWm$lowWmspan,], "RQHONFWNFLow")
createRQHrow(outputFile, resultsOnlyWm[resultsOnlyWm$treatmentCombination=="WF-WNF" & resultsOnlyWm$lowWmspan,], "RQHWFWNFLow")

resultsLong$treatmentR <- relevel(resultsLong$treatmentR, ref="WNF")
modelONFWNF <- lmer(efficiencyR ~ treatmentR + reviewNumber + (1|pId) + (1|changeR), data = droplevels(resultsLong[resultsLong$treatmentCombination=="ONF-WNF",]))
modelONFWNFci <- confint(modelONFWNF, method="boot")
writeLatexValue(outputFile, "RQHONFWNFAllLmerEffect", coef(modelONFWNF)$pId$treatmentRONF[1])
writeLatexValue(outputFile, "RQHONFWNFAllLmerEffectCiLow", modelONFWNFci["treatmentRONF",1])
writeLatexValue(outputFile, "RQHONFWNFAllLmerEffectCiHigh", modelONFWNFci["treatmentRONF",2])
writeLatexValue(outputFile, "RQHONFWNFAllLmerRSqMarg", r.squaredGLMM(modelONFWNF)[,"R2m"])
writeLatexValue(outputFile, "RQHONFWNFAllLmerRSqCond", r.squaredGLMM(modelONFWNF)[,"R2c"])

modelWFWNF <- lmer(efficiencyR ~ treatmentR + reviewNumber + (1|pId) + (1|changeR), data = droplevels(resultsLong[resultsLong$treatmentCombination=="WF-WNF",]))
modelWFWNFci <- confint(modelWFWNF, method="boot")
writeLatexValue(outputFile, "RQHWFWNFAllLmerEffect", coef(modelWFWNF)$pId$treatmentRWF[1])
writeLatexValue(outputFile, "RQHWFWNFAllLmerEffectCiLow", modelWFWNFci["treatmentRWF",1])
writeLatexValue(outputFile, "RQHWFWNFAllLmerEffectCiHigh", modelWFWNFci["treatmentRWF",2])
writeLatexValue(outputFile, "RQHWFWNFAllLmerRSqMarg", r.squaredGLMM(modelWFWNF)[,"R2m"])
writeLatexValue(outputFile, "RQHWFWNFAllLmerRSqCond", r.squaredGLMM(modelWFWNF)[,"R2c"])

resultsLong$treatmentR <- relevel(resultsLong$treatmentR, ref="WF")
modelOFWF <- lmer(efficiencyR ~ treatmentR + reviewNumber + (1|pId) + (1|changeR), data = droplevels(resultsLong[resultsLong$treatmentCombination=="OF-WF",]))
modelOFWFci <- confint(modelOFWF, method="boot")
writeLatexValue(outputFile, "RQHOFWFAllLmerEffect", coef(modelOFWF)$pId$treatmentROF[1])
writeLatexValue(outputFile, "RQHOFWFAllLmerEffectCiLow", modelOFWFci["treatmentROF",1])
writeLatexValue(outputFile, "RQHOFWFAllLmerEffectCiHigh", modelOFWFci["treatmentROF",2])
writeLatexValue(outputFile, "RQHOFWFAllLmerRSqMarg", r.squaredGLMM(modelOFWF)[,"R2m"])
writeLatexValue(outputFile, "RQHOFWFAllLmerRSqCond", r.squaredGLMM(modelOFWF)[,"R2c"])

writeLatexValue(outputFile, "RQHcomplicatedExpected", sum(results$comparisonComplicated == "expected"))
writeLatexValue(outputFile, "RQHcomplicatedSame", sum(results$comparisonComplicated == "same"))
writeLatexValue(outputFile, "RQHcomplicatedInverse", sum(results$comparisonComplicated == "inverse"))
writeLatexValue(outputFile, "RQHunderstandingExpected", sum(results$comparisonUnderstanding == "expected"))
writeLatexValue(outputFile, "RQHunderstandingSame", sum(results$comparisonUnderstanding == "same"))
writeLatexValue(outputFile, "RQHunderstandingInverse", sum(results$comparisonUnderstanding == "inverse"))


# threats to validty

writeLatexValue(outputFile, "ThrDropoutRO", length(dropoutR1$pId))
writeLatexValue(outputFile, "ThrDropoutRT", length(dropoutR2$pId))
writeLatexValue(outputFile, "ThrDropoutBoth", length(dropoutR1$pId) + length(dropoutR2$pId))
writeLatexValue(outputFile, "ThrDropoutWorse", length(dropoutR1[dropoutR1$treatmentR1 %in% c("WF", "WNF"),]$pId) + length(dropoutR2[dropoutR2$treatmentR2 %in% c("WF", "WNF"),]$pId))


close(outputFile)
